'use strict';

var addSemis = require('css-add-semis');
var upsertCss = require('./upsert-css');
var uniquifyName = require('./uniquify-name');

function isName(val) {
  return val && /^[a-zA-Z][a-zA-Z0-9-_ ]*$/.test(val);
}

function cmzNamed(name) {
  var comps = [name];
  var raw = [];

  // split parts into compositions & raw css

  for (var _len = arguments.length, parts = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    parts[_key - 1] = arguments[_key];
  }

  parts.filter(Boolean).forEach(function (p) {
    var group = isName(p) ? comps : raw;
    group.push(p);
  });

  var css = renderCss(name, raw);
  css && upsertCss(name, css);

  return comps.join(' ');
}

function renderCss(name, parts) {
  var selector = '.' + name;
  var output = '';

  var wrapped = [];
  var unwrapped = [];
  parts.forEach(function (part) {
    // replace name placeholders
    part = part.replace(/\?/g, name);

    // if no selector placeholder was given, we need to wrap it ourselves
    var isWrapped = part.indexOf('{') >= 0;
    var group = isWrapped ? wrapped : unwrapped;
    group.push(part);
  });

  if (unwrapped.length) {
    output += selector + ' {' + unwrapped.join('\n') + '}';

    if (wrapped.length) {
      output += '\n';
    }
  }

  if (wrapped.length) {
    // replace selector placeholders with the unique selector
    output += wrapped.map(function (part) {
      return part.replace(/&/g, selector);
    }).join('\n');
  }

  return addSemis(output);
}

var cmz = function cmz() {
  for (var _len2 = arguments.length, parts = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
    parts[_key2] = arguments[_key2];
  }

  return cmzNamed.apply(undefined, [uniquifyName()].concat(parts));
};
cmz.named = cmzNamed;
cmz.reset = uniquifyName.reset;
cmz.sheets = upsertCss.sheets;

module.exports = cmz;